import java.util.List;
import java.util.ArrayList;
import java.io.FileWriter;
import java.io.IOException;

/**
 * Records telemetry data for the LunarLander class
 *
 * @author Michael Yu
 */
public class DataRecorder {

    private LunarLander lander;
    private List times;
    private List positions;
    private List velocities;
    private List rotations;
    private List throttles;

    static final private int INITIAL_SIZE = 2000;

    /**
     * Constructor.  Initialize all the lists and take an initial measurement.
     * Time is measured relative to when this constructor is called.
     *
     * @param lander is the lunar lander that this data recorder is "installed"
     *               on
     */
    public DataRecorder(LunarLander lander) {
        this.lander = lander;

        times = new ArrayList(INITIAL_SIZE);
        times.add(new Double(0));

        positions = new ArrayList(INITIAL_SIZE);
        positions.add(lander.getPosition());

        velocities = new ArrayList(INITIAL_SIZE);
        velocities.add(lander.getVelocity());

        rotations = new ArrayList(INITIAL_SIZE);
        rotations.add(new Double(lander.getAngle()));

        throttles = new ArrayList(INITIAL_SIZE);
        throttles.add(new Double(lander.getThrottle()));
    }

    /**
     * Record telemetry data from the lunar lander
     *
     * @param dt is the amount of time elapsed since the last measurement was
     *           taken
     */
    public void record(double dt) {
        Double previousTime = (Double) times.get(times.size() - 1);
        times.add(new Double(previousTime.doubleValue() + dt));
        positions.add(lander.getPosition());
        velocities.add(lander.getVelocity());
        rotations.add(new Double(lander.getAngle()));
        throttles.add(new Double(lander.getThrottle()));
    }

    /**
     * Record all telemetry data to a file.  The format is csv, with each line
     * representing a recording sample.  The data is written in the following
     * order: time, positionX, positionY, velocityX, velocityY, rotation,
     * throttle
     *
     * @param filename is the name of the file to save telemetry data to
     * @throws IOException if something bad happened :-(
     */
    public void dumpToFile(String filename) throws IOException {
        FileWriter writer = new FileWriter(filename);

        for (int i = 0; i < times.size(); i++) {
            writer.write(times.get(i) + ",");

            Vect2D position = (Vect2D) positions.get(i);
            writer.write(position.getX() + "," + position.getY() + ",");

            Vect2D velocity = (Vect2D) velocities.get(i);
            writer.write(velocity.getX() + "," + velocity.getY() + ",");

            writer.write(rotations.get(i) + ",");
            writer.write(throttles.get(i) + ",");
            writer.write("\n");
        }

        writer.close();
    }

    public List getTimes() {
        return times;
    }

    public List getPositions() {
        return positions;
    }

    public List getVelocities() {
        return velocities;
    }

    public List getRotations() {
        return rotations;
    }

    public List getThrottles() {
        return throttles;
    }
}
